﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using System.Xml.Serialization;
using System.IO;

namespace MaciejGrabek.WP7AccelerometerEmulatorUI.Logic
{
    public class RecordingProvider
    {
        private List<Vector3> _recorderItems;
        private bool isRecording = false;

        private int playbackIndex = 0;
        private int recordingLimit = 100;

        private bool _isPlaying = false;
        public bool IsPlaying { get { return _isPlaying; } }

        public RecordingProvider()
        {
            this._recorderItems = new List<Vector3>();              
        }

        public void Start(int limit)
        {
            this._recorderItems.Clear();
            this._isPlaying = false;
            this.recordingLimit = limit;
            this.isRecording = true;
        }

        public void Stop()
        {
            this._isPlaying = false;
            this.isRecording = false;
        }

        public void Save(string filePath)
        {
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(List<Vector3>));
                FileStream fs = new FileStream(filePath, FileMode.Create, FileAccess.ReadWrite);
                serializer.Serialize(fs, this._recorderItems);
                fs.Close();
            }
            catch (Exception ex) { }
        }

        public void Load(string filePath)
        {
            this._recorderItems.Clear();
            XmlSerializer serializer = new XmlSerializer(typeof(List<Vector3>));
            try
            {
                FileStream fs = new FileStream(filePath, FileMode.Open);
                this._recorderItems = (List<Vector3>)serializer.Deserialize(fs);
                fs.Close();
            }
            catch (Exception ex) { }

        }

        //used by play
        public Vector3 GetNextFromPlayback()
        {
            this.playbackIndex++;
            if (this.playbackIndex >= this._recorderItems.Count)
            {
                this.playbackIndex = 0;
                this._isPlaying = false;
            }

            return this._recorderItems[this.playbackIndex];
        }

        public void StartPlayback()
        {
            this.playbackIndex = 0;
            this._isPlaying = true;
        }

        public void ResetPlayback()
        {
            this._isPlaying = false;
            this.playbackIndex = 0;
        }

        public void AddRotationState(Vector3 state)
        {
            if (this.isRecording)
            {
                if (this._recorderItems.Count >= this.recordingLimit)
                {
                    this._recorderItems.RemoveAt(0);
                }

                this._recorderItems.Add(state);
            }
        }

        public void ClearPlayback()
        {
            this._recorderItems.Clear();
        }
    }
}
