﻿using System;
using System.Windows;
using System.Windows.Media.Media3D;
using System.Windows.Threading;
using MaciejGrabek.WP7AccelerometerEmulatorUI.Logic;
using Microsoft.Xna.Framework;

namespace MaciejGrabek.WP7AccelerometerEmulatorUI
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        AxisAngleRotation3D _rotationX, _rotationY, _rotationZ;

        DispatcherTimer _timer;
        Vector3 _currentForce;

        bool _isCalling = false;
        AccelerometerEmulatorProxyReference.AccelerometerEmulatorProxyClient client;

        RecordingProvider _recorder = new RecordingProvider();

        public MainWindow()
        {
            InitializeComponent();

            client = new AccelerometerEmulatorProxyReference.AccelerometerEmulatorProxyClient();

            InitializeRotations();
            InitializeAndRunTimer();
        }

        #region Initialization
        private void InitializeRotations()
        {
            _rotationX = new AxisAngleRotation3D(new Vector3D(1, 0, 0), 0);
            _rotationY = new AxisAngleRotation3D(new Vector3D(0, 1, 0), 0);
            _rotationZ = new AxisAngleRotation3D(new Vector3D(0, 0, 1), 0);
            Transform3D tx = new RotateTransform3D(_rotationX);
            Transform3D ty = new RotateTransform3D(_rotationY);
            Transform3D tz = new RotateTransform3D(_rotationZ);
            Transform3DGroup transformationGroup = new Transform3DGroup();
            //order is important
            transformationGroup.Children.Add(ty);
            transformationGroup.Children.Add(tx);
            transformationGroup.Children.Add(tz);
            this.phoneModel.Transform = transformationGroup;
        }

        private void InitializeAndRunTimer()
        {
            _timer = new DispatcherTimer();
            _timer.Interval = TimeSpan.FromMilliseconds(100);
            _timer.Tick += delegate
            {
                PerformSingleStep();
            };
            _timer.Start();
        } 
        #endregion

        #region View and comunication
        private void PerformSingleStep()
        {
            RefreshView();
            SendData();
        }

        private void RefreshView()
        {
            if (this._recorder.IsPlaying)
            {
                Vector3 recorded = this._recorder.GetNextFromPlayback();
                this.sliderX.Value = recorded.X;
                this.sliderY.Value = recorded.Y;
                this.sliderZ.Value = recorded.Z;
            }

            _rotationY.Angle = this.sliderY.Value;
            _rotationX.Angle = this.sliderX.Value;
            _rotationZ.Angle = this.sliderZ.Value;

            //int not float -> rounded value
            this._recorder.AddRotationState(new Vector3((int)this.sliderX.Value, (int)sliderY.Value, (int)sliderZ.Value));

            this.tbxX.Text = String.Format("X: {0:F2}", this.sliderX.Value);
            this.tbxY.Text = String.Format("Y: {0:F2}", this.sliderY.Value);
            this.tbxZ.Text = String.Format("Z: {0:F2}", this.sliderZ.Value);
            this._currentForce = GForceCalculator.Calculate((float)this.sliderX.Value,
                (float)this.sliderY.Value,
                (float)this.sliderZ.Value);


            this.tbxResult.Text = String.Format("X:{0:F2} Y:{1:F2} Z:{2:F2}", this._currentForce.X, this._currentForce.Y, this._currentForce.Z);
        }

        private void SendData()
        {
            if (!this._isCalling && this.cbxSendEnabled.IsChecked.HasValue && this.cbxSendEnabled.IsChecked.Value)
            {
                this._isCalling = true;
                try
                {
                    //change into async call
                    client.SetData(this._currentForce.X, this._currentForce.Y, this._currentForce.Z);
                    this._isCalling = false;
                }
                catch (Exception ex) { }
            }
        }
        #endregion

        #region Actions
        private void btnReset_Click(object sender, RoutedEventArgs e)
        {
            this.sliderX.Value = this.sliderY.Value = this.sliderZ.Value = 0;
        }
        #endregion

        #region recording
        
        private void btnStartRecording_Click(object sender, RoutedEventArgs e)
        {
            this.sliderX.IsEnabled = this.sliderY.IsEnabled = this.sliderZ.IsEnabled = true;

            this._recorder.Start(int.Parse(this.cbxRecordLimit.Text));
        }

        private void btnStopRecording_Click(object sender, RoutedEventArgs e)
        {
            this.sliderX.IsEnabled = this.sliderY.IsEnabled = this.sliderZ.IsEnabled = true;

            this._recorder.Stop();
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            this.sliderX.IsEnabled = this.sliderY.IsEnabled = this.sliderZ.IsEnabled = true;

            this._recorder.Stop();
            Microsoft.Win32.SaveFileDialog d = new Microsoft.Win32.SaveFileDialog();
            d.Filter = "XML File(*.xml)|*.xml";
            d.CheckFileExists = false;
            d.CheckPathExists = true;
            bool? result = d.ShowDialog();
            if (result.HasValue && result.Value)
            {
                this._recorder.Save(d.FileName);
            }
        }

        private void btnLoadRecording_Click(object sender, RoutedEventArgs e)
        {
            this.sliderX.IsEnabled = this.sliderY.IsEnabled = this.sliderZ.IsEnabled = true;

            this._recorder.Stop();
            Microsoft.Win32.OpenFileDialog d = new Microsoft.Win32.OpenFileDialog();
            d.CheckFileExists = true;
            d.CheckPathExists = true;
            d.Filter = "XML File(*.xml)|*.xml";
            bool? result = d.ShowDialog();
            if (result.HasValue && result.Value)
            {
                this._recorder.Load(d.FileName);
            }
        }

        private void btnPlayrecorded_Click(object sender, RoutedEventArgs e)
        {
            this._recorder.StartPlayback();
        }

        private void btnStopPlaying_Click(object sender, RoutedEventArgs e)
        {
            this._recorder.ResetPlayback();
        }

        private void btnClear_Click(object sender, RoutedEventArgs e)
        {
            this._recorder.ClearPlayback();
        }

        #endregion
    }
}
